@@ -34,7 +34,7 @@ no warnings qw(uninitialized);
 
 use Coro ();
 
-$VERSION = 0.53;
+$VERSION = 0.531;
 
 =item new [inital count]
 
@@ -77,10 +77,10 @@ sub timed_down {
       push @{$sem->[1]}, $Coro::current;
       Coro::schedule;
       if ($timeout) {
-         # ugly as hell. splice would be faster
-         for (0..$#{$_[0][1]}) {
-            if ($_[0][1][$_] == $Coro::current) {
-               splice @{$_[0][1]}, $_, 1;
+         # ugly as hell.
+         for (0..$#{$sem->[1]}) {
+            if ($sem->[1][$_] == $Coro::current) {
+               splice @{$sem->[1]}, $_, 1;
                return;
             }
          }
